## parameters
q.list.table <- c(20,50,75)
numb.designs <- 13
numb.rots <- 1 # number of different rot parameter settings. 
weight.rots <- 1

## function to data generation
data.gen <- function(n, prob = .2, cutoff = .1){ # prob and cutoff are related to forming H1
  z0 <- matrix(NA, nrow=n, nc=numb.designs) # data under null
  
  # design 1
  z0[,1] <- rnorm(n)
  for(i in 2:3){
    z0[,i] <- z0[,i-1]-1
  }
  
  # unif draw, for D 2,3
  u <- runif(n)
  
  # design 2
  v4 <- 2*rbeta(n,2,4)-1
  v8 <- -(2*rbeta(n,2,8)-1)
  z0[,4] <- v4*(u<1/3) + v8*(u>=1/3)
  z0[,5] <- v4
  
  # design 3
  X1 = rnorm(n,-1,1);
  X2 = rnorm(n,-0.2,0.2);
  X3 = rnorm(n,3,2.5);
  cuts = c(0.4,0.5);
  X  = (u<cuts[1])*X1 + (u<cuts[2])*(u>cuts[1])*X2 + (u>cuts[2])*X3;
  z0[,6] <- X
  rm(X1,X2,X3,cuts,X)

  p.list <- c(1/4, 1/10, 1/20)
  # design 4
  inv4 <- function(y,k) {(4/3*y-1)*(0<=y&y<=(3/4*(1-k))) +
    (2*k-sqrt(k*abs(3*k+6-8*y)))*((3/4*(1-k))<y & y <= ((3+k)/4)) +
      (4*y-3)*(((3+k)/4)<y & y<=1)
  }
  z0[,7:9] <- sapply(p.list, function(k) inv4(u,k))
 
  # design 5
  inv5 <- function(y,k) { (4/3*y-1)*(0<=y & y <= (3*(1-k)/4)) +
      (y*2+(k-3)/2)*((3*(1-k)/4)<y & y<=((k+3)/4)) +
      (y*4-3)*(((k+3)/4)<y & y<=1)
  }
  z0[,10:12] <- sapply(p.list, function(k) inv5(u,k))
  
  # design 7
  z0[,13] <- rnorm(n,0,.037)+sample(lee,n,replace=T)
  
  # alternative
  a <- cutoff
  p <- prob
  z1 <- z0 # data under alternative
  z1[z1<a & z1>0] <- sapply(z1[z1<a & z1>0], function(x) {x-2*x*rbinom(1,1,prob - 2*x)})
  
  list('z0'=z0, 'z1'=z1)
}

## function to run simulations and save raw results
simul <- function(n, mc, q.list, alpha.list){
  numb.alpha <- length(alpha.list)
  raw.res0 <- list('us' = array(NA, dim=c(mc, numb.designs, length(q.list)) ),
                   'mccrary' = array(NA, dim=c(mc,2, numb.designs), dimnames = list(NULL, c('p','N'), NULL)),
                   'cattaneo' = array(NA, dim=c(mc,3, numb.designs), dimnames = list(NULL, c('p','left-N','right-N'), NULL)),
                   'rot' = array(NA, dim=c(mc,  2, numb.designs, numb.rots, numb.alpha), dimnames = list(NULL,c('q-rot','test-stat'),NULL,NULL,alpha.list)))
  raw.res1 <- raw.res0
  set.seed(1)
  start <- Sys.time()
  for (j in 1:mc){
    x <- data.gen(n)
    z0 <- x$z0
    z1 <- x$z1
    
    raw.res0$us[j,,] <- sapply(q.list, function(y) apply(z0, 2, function(x) gtest.fn(x,y)))
    if(n==1e6){
      if(j<=1000){raw.res0$mccrary[j,,] <- apply(z0, 2, function(x) mccrary.fn(x))
                  raw.res0$cattaneo[j,,] <- apply(z0, 2, function(x) cjm.fn(x))}
    }else{
      raw.res0$mccrary[j,,] <- apply(z0, 2, function(x) mccrary.fn(x))
      raw.res0$cattaneo[j,,] <- apply(z0, 2, function(x) cjm.fn(x))
      }
    for(k in 1:numb.rots){
      for(a in 1:numb.alpha){
    raw.res0$rot[j,,,k,a] <-    apply(z0, 2, function(x){ rot <- q.rot(n, mean(x), sd(x), weight=weight.rots[k], alpha=alpha.list[a])$q.rot;
          Sn <- gtest.fn(x, rot);          return(c(rot,Sn))})}};rm(k,a)
    
    raw.res1$us[j,,] <- sapply(q.list, function(y) apply(z1, 2, function(x) gtest.fn(x,y)))
    if(n==1e6){ 
      if(j<=1000){raw.res1$mccrary[j,,] <- apply(z1, 2, function(x) mccrary.fn(x))
                  raw.res1$cattaneo[j,,] <- apply(z1, 2, function(x) cjm.fn(x))}
    }else{
      raw.res1$mccrary[j,,] <- apply(z1, 2, function(x) mccrary.fn(x))
      raw.res1$cattaneo[j,,] <- apply(z1, 2, function(x) cjm.fn(x))
    }
    for(k in 1:numb.rots){
      for(a in 1:numb.alpha){
      raw.res1$rot[j,,,k,a] <-    apply(z1, 2, function(x){ rot <- q.rot(n, mean(x), sd(x), weight=weight.rots[k], alpha=alpha.list[a])$q.rot;
      Sn <- gtest.fn(x, rot);          return(c(rot,Sn))})}};rm(k,a)
    
    # if((j %% 100) == 0){
      print(c(j, Sys.time()-start))
      # }
    
    rm(x,z0,z1)
  }; rm(j)
  
  save(raw.res0, raw.res1,n, file=paste0('raw_results',Sys.Date(),'_n=',n,'.RData'))}

## functions to implement tests
gtest.fn <- function(x,q) {sum(x[sort(sort.int(abs(x), index.return = T, method="quick")$ix[1:q])] > 0)} # function(x=data, q) that generates test statistic
mccrary.fn <- function(x){
  temp <- try(DCdensity(x, plot=F, ext.out=T), silent=T)
  if('try-error' %in% class(temp)) {rep(NA,2)} else {if(is.na(temp$theta)){rep(NA,2)}else{c(temp$p,sum(abs(x)<temp$bw))}}
}
cjm.fn <- function(x){
  # temp.jk <- try(rddensity(x, vce="jackknife", all = T), silent=T)
  temp.jk <- try(rddensity(x, vce="jackknife"), silent=T)
  # temp.pi <- try(rddensity(x, vce="plugin", all = T), silent = T)
  # if('try-error' %in% class(temp.jk)) temp.jk <- c(NA,NA) else temp.jk <- c(temp.jk$test$p_jk,temp.jk$test_p$p_jk)
  if('try-error' %in% class(temp.jk)) temp.jk <- rep(NA,3) else temp.jk <- c(temp.jk$test$p_jk, sum(x<0 & -x<temp.jk$h$left), sum(x>0 & x<temp.jk$h$right))
  # if('try-error' %in% class(temp.pi)) temp.pi <- c(NA,NA) else temp.pi <- c(temp.pi$test$p_asy,temp.pi$test_p$p_asy)
  c(temp.jk)
  # c(temp.jk,temp.pi)
}

## clunkly functions generating (c1,c2) and gamma (to satisfy size restriction for finite q)
BW <- function(q, alpha) {
  bw <- 1
  while(sum(dbinom(floor(q/2-bw):ceiling(q/2+bw), q, .5)) < 1-alpha){
    bw <- bw + 1
  }
  return(c(floor(q/2-bw), ceiling(q/2+bw))) }
GA <- function(q, alpha){(1-alpha-sum(dbinom((BW(q, alpha)[1]+1):(BW(q, alpha)[2]-1), q, .5)))/sum(dbinom(c(BW(q, alpha)[1],BW(q, alpha)[2]), q, .5))}

## function to generate result and na tables
results.fn <- function(raw.res0, raw.res1,  n,q.list, alpha, weight){
  if(!(alpha %in% alpha.list)){stop('Incorrect size (alpha)')}
  if(!(weight %in% weight.rots)){stop('Incorrect weight')}
  ## generate results (Pr(rej)) from raw results
  res.H0 <- matrix(NA, nr=2*length(q.list)+2, nc=numb.designs,
                   dimnames=list(c( sapply(q.list, function(x){paste0("R q=",x,sep="")}),sapply(q.list, function(x){paste0("NR q=",x,sep="")}),
                                    # "McCrary", "CJM-jk BC", "CJM-jk", "CJM-asy BC", "CJM-asy"), 
                                    "McCrary", "CJM-jk BC"), 
                                 c(paste0("D1: $\\mu=",-(0:2),"$"), paste0("D2: $\\lambda=",c(1,"\\frac{1}{3}"),"$"),paste0("D3"),
                                   paste0("D4: $\\kappa=",c('0.25','0.10','0.05'),"$"), paste0("D5: $\\kappa=",c('0.25','0.10','0.05'),"$"), "D6")))
  
  res.H1 <- res.H0 # for alternative
  
  # compute empirical size (gtest)
  for (q in q.list){
    temp <- rbinom(length(raw.res0$us[,1,1]), 1, 1-GA(q,alpha))
    apply(raw.res0$us[,,which(q.list==q)], 2, function(x) mean(abs(q/2-x)>(q/2-BW(q,alpha)[1]-temp)   ) ) -> res.H0[which(q.list==q),]
    apply(raw.res0$us[,,which(q.list==q)], 2, function(x) mean(abs(q/2-x)>(q/2-BW(q,alpha)[1])   ) ) -> res.H0[length(q.list)+which(q.list==q),]
    rm(temp)
  }
  for (q in q.list){
    temp <- rbinom(length(raw.res1$us[,1,1]), 1, 1-GA(q,alpha))
    apply(raw.res1$us[,,which(q.list==q)], 2, function(x) mean(abs(q/2-x)>(q/2-BW(q,alpha)[1]-temp)   ) ) -> res.H1[which(q.list==q),]
    apply(raw.res1$us[,,which(q.list==q)], 2, function(x) mean(abs(q/2-x)>(q/2-BW(q,alpha)[1])   ) ) -> res.H1[length(q.list)+which(q.list==q),]
    rm(temp)
  }
  
  # (gtest, rot)
  res.rot0 <- apply(raw.res0$rot[,,,which(weight.rots==weight),which(alpha.list==alpha)], 3, function(x) {
    temp <- t(sapply(x[,1], function(y) c(rbinom(1,1, 1-GA(y,alpha)), BW(y,alpha)[1])))
    cat_r <- abs(x[,1]/2-x[,2])>(x[,1]/2-apply(temp,1,sum)); 
    cat_nr <- abs(x[,1]/2-x[,2])>(x[,1]/2-temp[,2]); return(c(mean(cat_r),mean(cat_nr)))      })
  res.rot1 <- apply(raw.res1$rot[,,,which(weight.rots==weight),which(alpha.list==alpha)], 3, function(x) {
    temp <- t(sapply(x[,1], function(y) c(rbinom(1,1, 1-GA(y,alpha)), BW(y,alpha)[1])))
    cat_r <- abs(x[,1]/2-x[,2])>(x[,1]/2-apply(temp,1,sum)); 
    cat_nr <- abs(x[,1]/2-x[,2])>(x[,1]/2-temp[,2]); return(c(mean(cat_r),mean(cat_nr)))      })
  # (existing tests)
  res.H0[2*length(q.list)+1,] <- apply(raw.res0$mccrary[,1,], 2, function(x) mean(x[!is.na(x)] < alpha))
  res.H0[-(1:(2*length(q.list)+1)),] <- t(apply(raw.res0$cattaneo[,1,], 2, function(x) mean(x[!is.na(x)] < alpha)))
  res.H1[2*length(q.list)+1,] <- apply(raw.res1$mccrary[,1,], 2, function(x) mean(x[!is.na(x)] < alpha))
  res.H1[-(1:(2*length(q.list)+1)),] <- t(apply(raw.res1$cattaneo[,1,], 2, function(x) mean(x[!is.na(x)] < alpha)))
  
  # format data for table
  res.H0 <- format(round(res.H0, 3)*100, nsmall = 1) 
  res.H1 <- format(round(res.H1, 3)*100, nsmall = 1) 
  res.rot0 <- format(round(res.rot0, 3)*100, nsmall = 1) 
  res.rot1 <- format(round(res.rot1, 3)*100, nsmall = 1)
  
  res.H0.table <- cbind(colnames(res.H0),t(res.H0[which(q.list %in% q.list.table)+length(q.list),]),
                        res.rot0[2,], '',
                        res.rot0[1,], '',
                        t(res.H0[2*length(q.list)+c(1,2),]) )
  
  res.H1.table <- cbind(colnames(res.H0),t(res.H1[which(q.list %in% q.list.table)+length(q.list),]),
                        res.rot1[2,], '',
                        res.rot1[1,], '',
                        t(res.H1[2*length(q.list)+c(1,2),]) )
  
  colnames(res.H0.table) <- c("Design", 20, 50 ,75, '$\\hat{q}_{irot}$', ' ', '$\\hat{q}_{irot}$', ' ', ' ', ' ')
  colnames(res.H1.table) <- colnames(res.H0.table)
  
  res.final.table <- cbind(res.H0.table[,c(1:7,9:10,rep(8,2))], res.H1.table[,c(2:7,9:10)]);
  rm(res.H0.table,res.H1.table)
  
  addtorow <- list()
  addtorow$pos <- list(-1)
  addtorow$command <- "\\hline\\hline
  \\multicolumn{1}{c}{}  & \\multicolumn{8}{c}{Rejection Rate under $H_0$}  & \\multicolumn{1}{c}{} & & \\multicolumn{8}{c}{Rejection Rate under $H_1$}\\\\ 
  \\multicolumn{1}{c}{}  & \\multicolumn{4}{c}{AS-NR} & & AS-R & McC & \\multicolumn{1}{c}{CJM} & \\multicolumn{1}{c}{} & & \\multicolumn{4}{c}{AS-NR} & & AS-R & McC & CJM \\\\
  \\multicolumn{1}{c}{}   & \\multicolumn{4}{c}{$q$} & & $q$ &  &\\multicolumn{1}{c}{} & \\multicolumn{1}{c}{} & \\multicolumn{1}{c}{} & \\multicolumn{4}{c}{$q$} & & $q$ &  & \\\\
  \\cline{2-5} \\cline{7-7} \\cline{12-15} \\cline{17-17}
  \\multicolumn{1}{c}{Design} & 20 & 50 & 75 & $\\hat q_{\\rm irot}$ & & $\\hat q_{\\rm irot}$ & &\\multicolumn{1}{c}{} &\\multicolumn{1}{c}{} &\\multicolumn{1}{c}{} & 20 & 50 & 75 & $\\hat q_{\\rm irot}$ & & $\\hat q_{\\rm irot}$ & & \\\\"
  
  if(alpha==.1 & n==1000){table.name <- 'Table1'}
  if(alpha==.1 & n==5000){table.name <- 'Table2'}
  if(alpha==.1 & n==100000){table.name <- 'SuppTable1'}
  if(alpha==.1 & n==1000000){table.name <- 'SuppTable2'}
  if(alpha==.05 & n==1000){table.name <- 'SuppTable3'}
  if(alpha==.05 & n==5000){table.name <- 'SuppTable4'}
  if(alpha==.01 & n==1000){table.name <- 'SuppTable5'}
  if(alpha==.01 & n==5000){table.name <- 'SuppTable6'}
  
  print(xtable(res.final.table, align=c('l','l',rep('c',18))), sanitize.text.function = function(x) {x},include.rownames=FALSE, include.colnames=FALSE ,floating=F, size='small',
        hline.after=c(0,3,5,6,09,12,13,13), add.to.row = addtorow , file=paste0('./results/',table.name,'.tex'))
  
  ## computes mean "eff sample size"
  if(alpha==.1 & (n==1000 | n==5000)){
    qrot <- apply(raw.res0$rot[,1,,which(weight.rots==weight),which(alpha.list==alpha)], 2, mean);
    eff.mcr <- apply(raw.res0$mccrary[,2,], 2, function(x)mean(x, na.rm = T))
    eff.cat <- apply(apply(raw.res0$cattaneo[,-1,], c(1,3), sum), 2, function(x)mean(x, na.rm = T))
    ess <- rbind(qrot,eff.mcr, eff.cat)
  save(ess, file=paste0('qrot_results',Sys.Date(),'n=',n,'.RData'))}
  
  ## outputs data for plots
  if(alpha==.1 & n==1000){
  data.plots <- cbind(q.list,matrix(res.H0[1:(nrow(res.H0)-2),c(1,13,3,6)],nr=length(q.list)))
  colnames(data.plots) <- c('q',	'RD1',	"NRD1",	'RD6',	'NRD6',	'RD12',	'NRD12'	,'RD3'	,'NRD3')
  write.csv(data.plots, file='./results/dataplots.csv', row.names = F)
  
  data.plots.FULL <- cbind(q.list,matrix(res.H0[1:(nrow(res.H0)-2),],nr=length(q.list)))
  colnames(data.plots.FULL) <- c('q',	paste0(rep(c('R_','NR_'),numb.designs),rep(colnames(res.H0),each=2)))
  write.csv(data.plots.FULL, file='./results/dataplotsFULL.csv', row.names = F)}
  
  
  # ## counts occasions that cattaneo/mccrary tests return NA
  # NA.H0 <- cbind(apply(raw.res0$mccrary, 2, function(x) sum(is.na(x))),
  #                apply(raw.res0$cattaneo, 2, function(x) sum(is.na(x))))
  # 
  # NA.H1 <- cbind(apply(raw.res1$mccrary, 2, function(x) sum(is.na(x))),
  #                apply(raw.res1$cattaneo, 2, function(x) sum(is.na(x))))
  # 
  # colnames(NA.H0) <- c("McCrary", "CJM-jk BC")
  # rownames(NA.H0) <- rownames(res.H0.table)
  # colnames(NA.H1) <- colnames(NA.H0);  rownames(NA.H1) <- rownames(NA.H0)

}

## required packages
require(rdd) # package and supporting packages for McCrary test ("?DCdensity")
require(rddensity) # Cattaneo test

require(knitr) # latex tables
require(kableExtra) # nice latex tables
require(xtable)

